// Implementation of the Lookup Service interface used to send call information
// to the client.
// Authors:
// Cameron Hayes
// Mac ...
// Elias Sarraf
unit Lookup.ServiceImpl;

interface

uses
  XData.Server.Module,
  XData.Service.Common,
  Database, Data.DB,
  Lookup.Service, System.Hash, System.Classes;


type
  [ServiceImplementation]
  TLookupService = class(TInterfacedObject, ILookupService)
  strict private
  ordersDB: TFDatabaseModule;
  private
    function GetItems(searchOptions: string): TItemList;
    function GetUsers(searchOptions: string): TUserList;
    function GetOrders(searchOptions: string): TOrderList;
    function EditUser(const editOptions: string): string;
    function Search(phoneNum: string): TOrderList;
    function AddUser(userInfo: string): string;
    function AddItem(itemInfo: string): string;
    function DelUser(username: string): string;
    procedure AfterConstruction; override;
    procedure BeforeDestruction; override;
  end;

implementation

uses
  System.SysUtils,
  System.Generics.Collections,
  XData.Sys.Exceptions, uLibrary;

procedure TLookupService.AfterConstruction;
begin
  inherited;
  ordersDB := TFDatabaseModule.Create(nil);

end;

procedure TLookupService.BeforeDestruction;
begin
  ordersDB.Free;
  inherited;
end;

function TLookupService.Search(phoneNum: string): TOrderList;
// Searchs the database for a specific from phone number then returns a list of
// calls from said number
// SQL: String of the sql query used to retrieve info from the database
// call: object representing the call information we want to send to the client
// Return: A list of calls
var
SQL: string;
order: TOrderItem;
begin
  SQL := 'select * ' +
          'from calls inner join recordings on calls.sid = recordings.call_sid ' +
         'where from_formatted = ' + QuotedStr(PhoneNum)  + ' order by calls.date_created desc';

  doQuery(ordersDB.UniQuery1, SQL);

  Result:= TOrderList.Create;
  Result.data := TList<TOrderItem>.Create;
  TXDataOperationContext.Current.Handler.ManagedObjects.Add( Result.data );

  while not ordersDB.UniQuery1.Eof do
    begin
      order := TOrderItem.Create;
      TXDataOperationContext.Current.Handler.ManagedObjects.Add( order );
      Result.data.Add( order );
      //TODO
    end;
    ordersDB.UniQuery1.Close;
    SQL:= '';

    doQuery(ordersDB.UniQuery1, SQL);

    Result.count := ordersDB.UniQuery1.FieldByName('total_count').AsInteger;

end;

function TLookupService.GetOrders(searchOptions: string): TOrderList;
var
 params: TStringList;
  PageNum: integer;
  PageSize: integer;
  OrderBy: string;
  offset: string;
  limit: string;
  SQL: string;
  Order: TOrderItem;
  colors: string;
  ColorType: string;
begin
  params := TStringList.Create;
  params.StrictDelimiter := true;
  // parse the searchOptions
  params.Delimiter := '&';
  params.DelimitedText := searchOptions;

  PageNum := StrToInt(params.Values['pagenumber']);
  PageSize := StrToInt(params.Values['pagesize']);
  OrderBy := params.Values['orderby'];

  offset := IntToStr((PageNum - 1) * PageSize);
  limit := IntToStr(PageSize);

  SQL := 'SELECT o.ORDER_ID, o.LOCATION AS Loc, c.NAME AS COMPANY_NAME, o.JOB_NAME, o.ORDER_DATE, o.ORDER_TYPE, ' +
       '(SELECT oss.STATUS_DATE ' +
       ' FROM orders_status_schedule oss ' +
       ' WHERE oss.ORDER_ID = o.ORDER_ID AND oss.ORDER_STATUS = ''PROOF'') AS PROOF_DUE, ' +
       '(SELECT os.STATUS_TIMESTAMP ' +
       ' FROM orders_status os ' +
       ' WHERE os.ORDER_ID = o.ORDER_ID AND os.ORDER_STATUS = ''PROOF'' ' +
       ' ORDER BY os.STATUS_TIMESTAMP DESC LIMIT 1) AS PROOF_DONE, ' +
       '(SELECT oss.STATUS_DATE ' +
       ' FROM orders_status_schedule oss ' +
       ' WHERE oss.ORDER_ID = o.ORDER_ID AND oss.ORDER_STATUS = ''ART'') AS ART_DUE, ' +
       '(SELECT os.STATUS_TIMESTAMP ' +
       ' FROM orders_status os ' +
       ' WHERE os.ORDER_ID = o.ORDER_ID AND os.ORDER_STATUS = ''ART'' ' +
       ' ORDER BY os.STATUS_TIMESTAMP DESC LIMIT 1) AS ART_DONE, ' +
       '(SELECT oss.STATUS_DATE ' +
       ' FROM orders_status_schedule oss ' +
       ' WHERE oss.ORDER_ID = o.ORDER_ID AND oss.ORDER_STATUS = ''PLATE'') AS PLATE_DUE, ' +
       '(SELECT os.STATUS_TIMESTAMP ' +
       ' FROM orders_status os ' +
       ' WHERE os.ORDER_ID = o.ORDER_ID AND os.ORDER_STATUS = ''PLATE'' ' +
       ' ORDER BY os.STATUS_TIMESTAMP DESC LIMIT 1) AS PLATE_DONE, ' +
       '(SELECT oss.STATUS_DATE ' +
       ' FROM orders_status_schedule oss ' +
       ' WHERE oss.ORDER_ID = o.ORDER_ID AND oss.ORDER_STATUS = ''MOUNT'') AS MOUNT_DUE, ' +
       '(SELECT os.STATUS_TIMESTAMP ' +
       ' FROM orders_status os ' +
       ' WHERE os.ORDER_ID = o.ORDER_ID AND os.ORDER_STATUS = ''MOUNT'' ' +
       ' ORDER BY os.STATUS_TIMESTAMP DESC LIMIT 1) AS MOUNT_DONE, ' +
       '(SELECT oss.STATUS_DATE ' +
       ' FROM orders_status_schedule oss ' +
       ' WHERE oss.ORDER_ID = o.ORDER_ID AND oss.ORDER_STATUS = ''SHIP'') AS SHIP_DUE, ' +
       '(SELECT os.STATUS_TIMESTAMP ' +
       ' FROM orders_status os ' +
       ' WHERE os.ORDER_ID = o.ORDER_ID AND os.ORDER_STATUS = ''SHIP'' ' +
       ' ORDER BY os.STATUS_TIMESTAMP DESC LIMIT 1) AS SHIP_DONE, ' +
       'o.PRICE, qb.QB_REF_NUM ' +
       'FROM orders o ' +
       'JOIN customers c ON c.CUSTOMER_ID = o.COMPANY_ID ' +
       'LEFT JOIN qb_sales_orders qb ON qb.ORDER_ID = o.ORDER_ID ' +
       'ORDER BY o.ORDER_DATE DESC ' +
       'LIMIT ' + limit + ' OFFSET ' + offset;

  doQuery(ordersDB.UniQuery1, SQL);

  Result:= TOrderList.Create;
  Result.data := TList<TOrderItem>.Create;
  TXDataOperationContext.Current.Handler.ManagedObjects.Add( Result.data );
  while not ordersDB.UniQuery1.Eof do
    begin
      Order := TOrderItem.Create;
      TXDataOperationContext.Current.Handler.ManagedObjects.Add( Order );
      Result.data.Add( Order );
      order.ID := ordersDB.UniQuery1.FieldByName('ORDER_ID').AsString;
      order.companyName := ordersDB.UniQuery1.FieldByName('COMPANY_NAME').AsString;
      order.jobName := ordersDB.UniQuery1.FieldByName('JOB_NAME').AsString;
      order.orderDate := ordersDB.UniQuery1.FieldByName('ORDER_DATE').AsString;
      order.proofDue := ordersDB.UniQuery1.FieldByName('PROOF_DUE').AsString;
      order.proofDone := ordersDB.UniQuery1.FieldByName('PROOF_DONE').AsString;
      order.artDue := ordersDB.UniQuery1.FieldByName('ART_DUE').AsString;
      order.artDone := ordersDB.UniQuery1.FieldByName('ART_DONE').AsString;
      order.mountDue := ordersDB.UniQuery1.FieldByName('MOUNT_DUE').AsString;
      order.mountDone := ordersDB.UniQuery1.FieldByName('MOUNT_DONE').AsString;
      order.shipDue := ordersDB.UniQuery1.FieldByName('SHIP_DUE').AsString;
      order.shipDone := ordersDB.UniQuery1.FieldByName('SHIP_DONE').AsString;
      order.price := ordersDB.UniQuery1.FieldByName('PRICE').AsString;
      order.qbRefNum := ordersDB.UniQuery1.FieldByName('QB_REF_NUM').AsString;

      if ordersDB.UniQuery1.FieldByName('ORDER_TYPE').AsString = 'web_plate' then
      begin
        colorType := 'quantity_and_colors_qty_colors';
        SQL := 'Select quantity_and_colors_qty_colors from web_plate_orders where order_id = ' + order.ID;
      end
      else
      begin
        colorType := 'colors_colors';
        SQL := 'Select colors_colors from corrugated_plate_orders where order_id = ' + order.ID;
      end;



      doQuery(ordersDB.UniQuery2, SQL);
      colors := ordersDB.UniQuery2.FieldByName(ColorType).AsString;
      order.colors := colors;

      ordersDB.UniQuery1.Next;
    end;

    ordersDB.UniQuery1.Close;
    SQL := 'SELECT COUNT(*) AS total_count '+
            'FROM orders o ' +
            'JOIN customers c ON c.CUSTOMER_ID = o.COMPANY_ID '  +
            'LEFT JOIN qb_sales_orders qb ON qb.ORDER_ID = o.ORDER_ID';

    doQuery(ordersDB.UniQuery1, SQL);
    Result.count := ordersDB.UniQuery1.FieldByName('total_count').AsInteger;

    ordersDB.UniQuery1.Close;
end;

function TLookupService.GetItems(searchOptions: string): TItemList;
var
  params: TStringList;
  PageNum: integer;
  PageSize: integer;
  OrderBy: string;
  offset: string;
  limit: string;
  SQL: string;
  item: TItemItem;
begin
  params := TStringList.Create;
  params.StrictDelimiter := true;
  // parse the searchOptions
  params.Delimiter := '&';
  params.DelimitedText := searchOptions;

  PageNum := StrToInt(params.Values['pagenumber']);
  PageSize := StrToInt(params.Values['pagesize']);
  OrderBy := params.Values['orderby'];

  offset := IntToStr((PageNum - 1) * PageSize);
  limit := IntToStr(PageSize);

  SQL := 'select * from qb_items ' + 'limit ' + limit + ' offset ' + offset;

  doQuery(ordersDB.UniQuery1, SQL);

  Result:= TItemList.Create;
  Result.data := TList<TItemItem>.Create;
  TXDataOperationContext.Current.Handler.ManagedObjects.Add( Result.data );

  while not ordersDB.UniQuery1.Eof do
    begin
      item := TItemItem.Create;
      TXDataOperationContext.Current.Handler.ManagedObjects.Add( item );
      Result.data.Add( item );
      item.ID := ordersDB.UniQuery1.FieldByName('qb_items_id').AsString;
      item.name := ordersDB.UniQuery1.FieldByName('qb_item_name').AsString;
      item.description := ordersDB.UniQuery1.FieldByName('item_desc').AsString;
      item.status := ordersDB.UniQuery1.FieldByName('status').AsString;

      ordersDB.UniQuery1.Next;
    end;
    ordersDB.UniQuery1.Close;
    SQL:= 'select count(*) as total_count from qb_items';
    doQuery(ordersDB.UniQuery1, SQL);
    Result.count := ordersDB.UniQuery1.FieldByName('total_count').AsInteger;
    ordersDB.UniQuery1.Close;
end;

function TLookupService.GetUsers(searchOptions: string): TUserList;
var
SQL: string;
user: TUserItem;
begin
  if searchOptions = '' then
    SQL := 'select * from users order by NAME ASC'
  else
    SQL := 'select * from users where username=' + quotedStr(searchOptions);
  doQuery(ordersDB.UniQuery1, SQL);

  Result:= TUserList.Create;
  Result.data := TList<TUserItem>.Create;
  TXDataOperationContext.Current.Handler.ManagedObjects.Add( Result.data );

  while not ordersDB.UniQuery1.Eof do
    begin
      user := TUserItem.Create;
      TXDataOperationContext.Current.Handler.ManagedObjects.Add( user );
      Result.data.Add( user );
      user.userID := ordersDB.UniQuery1.FieldByName('USER_ID').AsString;
      user.username := ordersDB.UniQuery1.FieldByName('USER_NAME').AsString;
      user.password := ordersDB.UniQuery1.FieldByName('PASSWORD').AsString;
      user.full_name := ordersDB.UniQuery1.FieldByName('NAME').AsString;
      user.status := ordersDB.UniQuery1.FieldByName('STATUS').AsString;
      user.email_address := ordersDB.UniQuery1.FieldByName('EMAIL').AsString;
      user.AType := ordersDB.UniQuery1.FieldByName('ACCESS_TYPE').AsString;
      user.rights := ordersDB.UniQuery1.FieldByName('SYSTEM_RIGHTS').AsInteger;
      user.perspectiveID := ordersDB.UniQuery1.FieldByName('PERSPECTIVE_ID').AsString;
      user.QBID := ordersDB.UniQuery1.FieldByName('QB_ID').AsString;
      ordersDB.UniQuery1.Next;
    end;
    ordersDB.UniQuery1.Close;
    SQL:= 'select count(*) as total_count from users';
    doQuery(ordersDB.UniQuery1, SQL);
    Result.count := ordersDB.UniQuery1.FieldByName('total_count').AsInteger;
    ordersDB.UniQuery1.Close;
end;

function TLookupService.EditUser(const editOptions: string): string;
var
params: TStringList;
user: string;
password: string;
full_name: string;
status: string;
email: string;
access: string;
rights: string;
perspective: string;
QB: string;
SQL: string;
newUser: string;
hashString: string;
hashPW: string;


begin
  params := TStringList.Create;
  // parse the searchOptions
  params.Delimiter := '&';
  params.StrictDelimiter := true;
  params.DelimitedText := editOptions;
  user := params.Values['username'];
  password := params.Values['password'];
  full_name := params.Values['fullname'];
  status := params.Values['status'];
  email := params.Values['email'];
  access := params.Values['access'];
  rights := params.Values['rights'];
  perspective := params.Values['perspective'];
  QB := params.Values['QB'];
  newUser := params.Values['newuser'];

  SQL := 'select * from users where USER_NAME = ' + QuotedStr(user);
  doQuery(ordersDB.UniQuery1, SQL);

  if ordersDB.UniQuery1.IsEmpty then
    Result := 'Failure:No such user found'
  else
  begin
    ordersDB.UniQuery1.Edit;

    //user.password := ordersDB.UniQuery1.FieldByName('PASSWORD').AsString;

    if (not (newUser.IsEmpty)) then
      ordersDB.UniQuery1.FieldByName('USER_NAME').AsString := newUser;

    if (not (full_name.IsEmpty)) then
      ordersDB.UniQuery1.FieldByName('NAME').AsString := full_name;

    if (not (status.IsEmpty)) then
    begin
      if( StrToBool(status) ) then
        ordersDB.UniQuery1.FieldByName('STATUS').AsString := 'ACTIVE'
      else
        ordersDB.UniQuery1.FieldByName('STATUS').AsString := 'INACTIVE'
    end;

    if (not (email.IsEmpty)) then
      ordersDB.UniQuery1.FieldByName('EMAIL').AsString := email;

    if (not (access.IsEmpty)) then
      ordersDB.UniQuery1.FieldByName('ACCESS_TYPE').AsString := Access;

    if (not (rights.IsEmpty)) then
      ordersDB.UniQuery1.FieldByName('SYSTEM_RIGHTS').AsInteger := StrToInt(rights);

    if (not (perspective.IsEmpty)) then
      ordersDB.UniQuery1.FieldByName('PERSPECTIVE_ID').AsString := perspective;

    if (not (QB.IsEmpty)) then
      ordersDB.UniQuery1.FieldByName('QB_ID').AsString := QB;


    {if((not (Password = 'hidden')) and (not (Password.IsEmpty))) then
    begin
      hashString :=  ordersDB.UniQuery1.FieldByName('date_created').AsString + password;
      hashPW := THashSHA2.GetHashString(hashString, THashSHA2.TSHA2Version.SHA512).ToUpper;
      ordersDB.UniQuery1.FieldByName('password').AsString := hashPW;
    end;}

    ordersDB.UniQuery1.Post;
    Result := 'Success:Edit Successful';
  end;
  ordersDB.UniQuery1.Close;
end;


function TLookupService.AddUser(userInfo:string): string;
var
user: string;
password: string;
full_name: string;
status: string;
email: string;
access: string;
rights: string;
perspective: string;
QB: string;
SQL: string;
dateCreated: TDateTime;
hashString: string;
hashPW: string;
params: TStringList;
begin
  params := TStringList.Create;
  params.StrictDelimiter := true;
  // parse the searchOptions
  params.Delimiter := '&';
  params.DelimitedText := userInfo;
  dateCreated := now;
  user := params.Values['username'];
  password := params.Values['password'];
  full_name := params.Values['fullname'];
  status := params.Values['status'];
  email := params.Values['email'];
  access := params.Values['access'];
  rights := params.Values['rights'];
  perspective := params.Values['perspective'];
  QB := params.Values['QB'];
  //newUser := params.Values['newuser'];
  //hashString :=  DateTimeToStr(dateCreated) + params.Values['password'];
  //hashPW := THashSHA2.GetHashString(hashString, THashSHA2.TSHA2Version.SHA512).ToUpper;
  SQL := 'select * from users where USER_NAME = ' + QuotedStr(params.Values['username'].toLower);
  ordersDB.UniQuery1.Close;
  ordersDB.UniQuery1.SQL.Text := SQL;
  ordersDB.UniQuery1.Open;
  if ordersDB.UniQuery1.IsEmpty then
  begin
    ordersDB.UniQuery1.Insert;

    ordersDB.UniQuery1.FieldByName('USER_NAME').AsString := user;
    ordersDB.UniQuery1.FieldByName('PASSWORD').AsString := password;
    //THashSHA2.GetHashString(hashString, THashSHA2.TSHA2Version.SHA512).ToUpper;
    //ordersDB.UniQuery1.FieldByName('date_created').AsString := DateTimeToStr(dateCreated);
    ordersDB.UniQuery1.FieldByName('NAME').AsString := full_name;

    if( StrToBool(status) ) then
      ordersDB.UniQuery1.FieldByName('STATUS').AsString := 'ACTIVE'
    else
      ordersDB.UniQuery1.FieldByName('STATUS').AsString := 'INACTIVE';

    ordersDB.UniQuery1.FieldByName('EMAIL').AsString := email;
    ordersDB.UniQuery1.FieldByName('ACCESS_TYPE').AsString := Access;
    ordersDB.UniQuery1.FieldByName('SYSTEM_RIGHTS').AsInteger := StrToInt(rights);
    ordersDB.UniQuery1.FieldByName('PERSPECTIVE_ID').AsString := perspective;
    ordersDB.UniQuery1.FieldByName('QB_ID').AsString := QB;

    ordersDB.UniQuery1.Post;
    Result := 'Success:User successfully added';
  end
  else
    Result := 'Failure:Username already taken';
end;

function TLookupService.AddItem(itemInfo: string): string;
var
params: TStringList;
Name: string;
Description: string;
Status: boolean;
SQL: string;
begin
  params := TStringList.Create;
  params.StrictDelimiter := true;
  // parse the searchOptions
  params.Delimiter := '&';
  params.DelimitedText := itemInfo;

  Name := params.Values['name'];
  Description := params.Values['description'];
  Status := StrToBool(params.Values['status']);

  SQL := 'select * from qb_items where qb_item_name = ' + QuotedStr(Name);

  doQuery(ordersDB.UniQuery1, SQL);
  if ordersDB.UniQuery1.IsEmpty then
  begin
    ordersDB.UniQuery1.Insert;

    ordersDB.UniQuery1.FieldByName('qb_item_name').AsString := Name;
    ordersDB.UniQuery1.FieldByName('item_desc').AsString := Description;
    if Status then
      ordersDB.UniQuery1.FieldByName('status').AsString := 'ACTIVE'
    else
      ordersDB.UniQuery1.FieldByName('status').AsString := 'INACTIVE';

    ordersDB.UniQuery1.Post;
    Result := 'Success:Item successfully added';
  end
  else
    Result := 'Failure: Item already exists';
end;

function TLookupService.DelUser(username: string): string;
var

SQL: string;
params: TStringList;
begin
  SQL := 'select * from users where username = ' + QuotedStr(username.toLower);
  ordersDB.UniQuery1.Close;
  ordersDB.UniQuery1.SQL.Text := SQL;
  ordersDB.UniQuery1.Open;
  if ordersDB.UniQuery1.IsEmpty then
  begin
    Result := 'Failure:User does not exist';
  end
  else
  begin
    SQL:= 'DELETE FROM users where username = ' +  QuotedStr(username.toLower);
    ordersDB.UniQuery1.SQL.Text := SQL;
    ordersDB.UniQuery1.ExecSQL;
    Result := 'Success:User deleted';
  end;
end;

initialization
  RegisterServiceType(TLookupService);

end.

